"""Use case 1: complexitiy measurements in large text collection.

Data can be downloaded from http://pizzachili.dcc.uchile.cl/texts/nlang.
Current version works with e.g. input_filename = 'english.50MB',
sample_size = 5, compute_std_deviation = False, subsample_size = 100 and
num_subsamples = 100 (in less than 3 minutes). It quickly runs out of memory 
with larger amounts of data, and the computation seems to take forever when 
compute_std_deviation = True (even with a small sample_size) and/or parameter
num_subsamples is set to a large value.
"""

from LTTL.Input import Input
from LTTL.Segmenter import tokenize, sample
from LTTL.Processor import Processor

import codecs
import re
import time

def main():

    # Parameters.
    input_filename = 'english.1024MB'
    sample_size = 500
    compute_std_deviation = True
    subsample_size = 100
    num_subsamples = 1000
    
    # Open file and read content...
    input_file = codecs.open(input_filename, encoding='latin1')
    file_content = input_file.read()
    input_file.close()

    # Create an LTTL Input object for it.
    my_input = Input(file_content)
    print("Imported %i characters" % len(file_content))

    # Segment data into ebooks.
    ebook_seg = tokenize(
        my_input,
        [(re.compile(r'This etext'), 'split')],
        auto_number_as='num',
    )
    print("Segmented text into %i ebooks" % len(ebook_seg))

    # Draw random sample.
    sampled_seg, _ = sample(ebook_seg, sample_size)
    print("Sampled %i random ebooks" % sample_size)

    # Segment ebooks into lines.
    line_seg = tokenize(sampled_seg, [(re.compile(r'.+'), 'tokenize')])
    print("Segmented ebooks into %i lines" % len(line_seg))

    # Segment lines into words.
    word_seg = tokenize(line_seg, [(re.compile(r'\w+'), 'tokenize')])
    print("Segmented lines into %i words" % len(word_seg))

    # Segment words into letters.
    letter_seg = tokenize(word_seg, [(re.compile(r'.'), 'tokenize')])
    print("Segmented words into %i letters" % len(letter_seg))

    processor = Processor()

    # Count average word length for each ebook...
    table = processor.length_in_context(
        units=letter_seg,
        averaging={
            'segmentation': word_seg,
            'std_deviation': compute_std_deviation,
        },
        contexts={
            'segmentation': sampled_seg,
            'annotation_key': 'num',
        }
    )
    print("Computed average word length")
    print(table.to_string())

    # Count (weighted) average perplexity per 100 word tokens for each ebook...
    table2 = processor.variety_in_context(
        units={
            'segmentation': word_seg,
            'weighting': True,
        },
        contexts={
            'segmentation': sampled_seg,
            'annotation_key': 'num',
        },
        apply_resampling=True,
        subsample_size=subsample_size,
        num_subsamples=num_subsamples,
    )
    print("Computed lexical diversity")
    print(table2.to_string())

if __name__ == '__main__':
    start_time = time.time()
    main()
    print("Total execution time: %.3fs seconds" % (time.time() - start_time))


