"""Use case 2: stringifying a large annotated segmentation.

Data can be downloaded from http://pizzachili.dcc.uchile.cl/texts/nlang.
Current version works with e.g. input_filename = 'english.50MB',
sample_size = 3 and output_mode = 'plain_text' (in less than a minute).
It quickly runs out of memory with larger amounts of data, and also when the
output_mode is set to 'html' (even with a sample_size of 1).
"""

from LTTL.Input import Input
from LTTL.Segmenter import tokenize, sample
from LTTL.Processor import Processor

import codecs
import re
import time

def main():

    # Parameters.
    input_filename = 'english.1024MB'
    output_filename = 'use_case_2_output.txt'
    sample_size = 500
    output_mode = 'html'    # alternative is 'plain_text'
    
    # Open file and read content...
    input_file = codecs.open(input_filename, encoding='latin1')
    file_content = input_file.read()
    input_file.close()

    # Create an LTTL Input object for it.
    my_input = Input(file_content)
    print("Imported %i characters" % len(file_content))

    # Segment data into ebooks.
    ebook_seg = tokenize(
        my_input,
        [(re.compile(r'This etext'), 'split')],
        auto_number_as='num',
    )
    print("Segmented text into %i ebooks" % len(ebook_seg))

    # Draw random sample.
    sampled_seg, _ = sample(ebook_seg, sample_size)
    print("Sampled %i random ebooks" % sample_size)

    # Segment ebooks into long vs short words.
    word_seg = tokenize(
        sampled_seg, 
        [
            (re.compile(r'\b\w{1,3}\b'), 'tokenize', {'word_type': 'short'}),
            (re.compile(r'\b\w{4,}\b'), 'tokenize', {'word_type': 'long'}),
        ]
    )
    print("Segmented lines into %i words" % len(word_seg))

    # Segment words into consonants vs vowels.
    letter_seg = tokenize(
        word_seg, 
        [
            (re.compile(r'\w'), 'tokenize', {'letter_type': 'consonant'}),
            (re.compile(r'[aeiouy]'), 'tokenize', {'letter_type': 'vowel'}),
        ],
        merge_duplicates=True,
    )
    print("Segmented words into %i letters" % len(letter_seg))

    # Create string representation of the letter segmentation...
    output_string = ''
    if output_mode == 'plain_text':
        str_format = (
            '%(num)s\t%(__num__)s\t%(__content__)s\t'
            '%(word_type)s\t%(letter_type)s'
        )
        output_string = letter_seg.to_string(formatting=str_format)
        print("Created plain text representation of letter segmentation")
    elif output_mode == 'html':
        output_string = letter_seg.to_html()
        print("Created html representation of letter segmentation")
    else:
        print('Unknown output_mode parameter, no output.')
    
    # Open file and write string to it...
    if len(output_string) > 1:
        output_file = codecs.open(output_filename, mode='w', encoding='utf-8')
        output_file.write(output_string)
        output_file.close()


if __name__ == '__main__':
    start_time = time.time()
    main()
    print("Total execution time: %.3fs seconds" % (time.time() - start_time))


