"""Use case 3: various manipulations of large segmentations.

Data can be downloaded from http://pizzachili.dcc.uchile.cl/texts/nlang.
Current version works with e.g. input_filename = 'english.50MB',
sample_size = 30 and min_freq = 50 (in less than a minute).
It quickly runs out of memory with larger amounts of data.
"""

from LTTL.Input import Input
from LTTL.Segmenter import tokenize, sample, threshold, intersect, select
from LTTL.Processor import Processor

import codecs
import re
import time

def main():

    # Parameters.
    input_filename = 'english.1024MB'
    sample_size = 500
    min_freq = 50
    
    # Open file and read content...
    input_file = codecs.open(input_filename, encoding='latin1')
    file_content = input_file.read()
    input_file.close()

    # Create an LTTL Input object for it.
    my_input = Input(file_content)
    print("Imported %i characters" % len(file_content))

    # Segment data into ebooks.
    ebook_seg = tokenize(
        my_input,
        [(re.compile(r'This etext'), 'split')],
        auto_number_as='num',
    )
    print("Segmented text into %i ebooks" % len(ebook_seg))

    # Draw random sample.
    sampled_seg, _ = sample(ebook_seg, sample_size)
    print("Sampled %i random ebooks" % sample_size)

    # Segment ebooks into lines.
    line_seg = tokenize(sampled_seg, [(re.compile(r'.+'), 'tokenize')])
    print("Segmented ebooks into %i lines" % len(line_seg))

    # Segment lines into words.
    word_seg = tokenize(line_seg, [(re.compile(r'\w+'), 'tokenize')])
    print("Segmented lines into %i words" % len(word_seg))

    # Draw second random sample (single ebook).
    sampled_seg2, _ = sample(ebook_seg, 1)
    print("Sampled 1 random ebook")

    # Segment it into lines.
    line_seg2 = tokenize(sampled_seg2, [(re.compile(r'.+'), 'tokenize')])
    print("Segmented it into %i lines" % len(line_seg2))

    # Segment lines into words.
    word_seg2 = tokenize(line_seg2, [(re.compile(r'\w+'), 'tokenize')])
    print("Segmented lines into %i words" % len(word_seg2))
    
    # Remove low frequency words.
    stopword_seg, _ = threshold(word_seg2, min_count=min_freq)
    print("Created segmentation with %i stopword tokens" % len(stopword_seg))

    # Exclude from first sample words from second sample.
    filtered_seg, _ = intersect(word_seg, stopword_seg, mode='exclude')
    print("Removed %i stopwords" % (len(word_seg) - len(filtered_seg)))
    
    # Remove from remaining words those that contain letter 'k'.
    filtered_seg2, _ = select(filtered_seg, re.compile(r'k'), mode='exclude')
    print("Removed %i words containing 'k'" % (len(filtered_seg) - len(filtered_seg2)))
    

if __name__ == '__main__':
    start_time = time.time()
    main()
    print("Total execution time: %.3fs seconds" % (time.time() - start_time))


