"""Use case 4: construction and manipulation of large tables.

Data can be downloaded from http://pizzachili.dcc.uchile.cl/texts/nlang.
Current version works with e.g. input_filename = 'english.50MB'
and sample_size = 10 (in less than 5 minutes). It quickly runs out of memory 
with larger amounts of data.
"""

from LTTL.Input import Input
from LTTL.Segmenter import tokenize, sample
from LTTL.Processor import Processor

import codecs
import re
import time

def main():

    # Parameters.
    input_filename = 'english.1024MB'
    sample_size = 500

    # Open file and read content...
    input_file = codecs.open(input_filename, encoding='latin1')
    file_content = input_file.read()
    input_file.close()

    # Create an LTTL Input object for it.
    my_input = Input(file_content)
    print("Imported %i characters" % len(file_content))

    # Segment data into ebooks.
    ebook_seg = tokenize(
        my_input,
        [(re.compile(r'This etext'), 'split')],
        auto_number_as='num',
    )
    print("Segmented text into %i ebooks" % len(ebook_seg))

    # Draw random sample.
    sampled_seg, _ = sample(ebook_seg, sample_size)
    print("Sampled %i random ebooks" % sample_size)

    # Segment ebooks into lines.
    line_seg = tokenize(sampled_seg, [(re.compile(r'.+'), 'tokenize')])
    print("Segmented ebooks into %i lines" % len(line_seg))

    # Segment lines into words.
    word_seg = tokenize(line_seg, [(re.compile(r'\w+'), 'tokenize')])
    print("Segmented lines into %i words" % len(word_seg))

    processor = Processor()
    
    # Build document-term table...
    table = processor.count_in_context(
        units={'segmentation': word_seg},
        contexts={
            'segmentation': sampled_seg,
            'annotation_key': 'num',
        }
    )
    print("Built document-term matrix")
    
    # Transpose.
    transposed_table = table.to_transposed()
    print("Transposed table")
    
    # Compute independence quotients.
    quotients = transposed_table.to_normalized(mode='quotients')
    print("Computed independence quotients")
    
    # Convert table to "weighted flat" (=sparse) crosstab format.
    weighted_flat_table = quotients.to_weighted_flat()
    print("Converted to weighted flat format")
    
    # Sort
    sorted_table = weighted_flat_table.to_sorted(key_col_id='__unit__')
    print("Sorted table")
    
    # Recast to "pivot" crosstab format.
    pivot_table = sorted_table.to_pivot()
    print("Recast to pivot table")
    
    # Convert to numpy array.
    np_table = pivot_table.to_numpy()
    print("Converted to numpy array")
 
    print(np_table)

if __name__ == '__main__':
    start_time = time.time()
    main()
    print("Total execution time: %.3fs seconds" % (time.time() - start_time))


